import subprocess
import sys
import os
from emod_api.schema import dtk_post_process_schema as dpps

def dtk_to_schema( path_to_binary ):
    """
    Runs /path/to/Eradication --get-schema --schema-path=schema.json and then post-processes the schema into something more useful.
    Error cases handled:
    - schema.json file already exists in cwd; does not overwrite. Asks users to move and retry.
    - Specified binary fails to run to completion.
    - Specified binary fails to produce a schema.json
    """
    if os.path.exists( "schema.json" ):
        print( "There is a file called 'schema.json' in the current working directory. Please move or delete it and try again." )
        sys.exit()
    try:
        subprocess.call( [ path_to_binary, "--get-schema", "--schema-path", "schema.json" ], stdout=open(os.devnull) )
    except Exception as ex:
        print( "Something bad happened while trying to run the Eradication binary." )
        print( str( ex ) )
        sys.exit()
    if os.path.exists( "schema.json" ) == False:
        print( "The specified EMOD executable failed to write 'schema.json'." )
        sys.exit()
    dpps.application("schema.json")
    print( "Wrote 'schema.json' file." )

if __name__ == "__main__":
    if len( sys.argv ) == 1:
        print( "Please specify path to Eradication binary/exe." )
    else:
        dtk_to_schema( sys.argv[1] )
